import{_ as r,a as o,o as _,c as l,e,b as n,f as c,g as t}from"./index-06XPMlP6.js";const i={},d={style:{"background-color":"#d2d2fa",height:"100vh",display:"grid","place-items":"center"}},u=e("h2",{style:{"text-align":"center"}},[t("欢迎使用"),e("a",{href:"https://github.com/xaoyaoo/PyWxDump.git"},"PyWxDump"),t("聊天记录查看工具! ")],-1),p=e("p",null,"如需提前体验更多功能，请开通超级vip",-1);function h(f,m){const s=o("el-button"),a=o("router-link");return _(),l("div",d,[u,e("h3",null,[n(a,{to:"/db_init"},{default:c(()=>[n(s,null,{default:c(()=>[t("重新设置数据来源")]),_:1})]),_:1})]),p])}const g=r(i,[["render",h]]);export{g as default};
