import{d as n,r,q as a,o as c,c as i,e as t,g as s,t as _,s as d}from"./index-06XPMlP6.js";const h={style:{"background-color":"#d2d2fa",height:"100vh",display:"grid","place-items":"center"}},l={style:{"text-align":"center"}},p=t("a",{href:"https://github.com/xaoyaoo/PyWxDump.git"},"PyWxDump",-1),g={style:{"text-align":"center"}},f=n({__name:"IndexView",setup(m){const o=r("");return a(()=>{d().then(e=>{o.value=e}).catch(e=>{console.error("Error fetching API version:",e)})}),(e,x)=>(c(),i("div",h,[t("h2",l,[s("欢迎使用"),p,s("聊天记录查看工具! "),t("p",g,"当前版本："+_(o.value),1)])]))}});export{f as _};
